@@ -38,14 +38,19 @@ import math
 import struct
 import zlib
 from io import BytesIO
-from typing import TYPE_CHECKING, Any, Dict, Optional, Tuple, Union, cast
+from typing import Any, Dict, List, Optional, Tuple, Union, cast
 
-from ._utils import b_, deprecate_with_replacement, ord_, paeth_predictor
+from ._utils import (
+    b_,
+    deprecate_with_replacement,
+    logger_warning,
+    ord_,
+    paeth_predictor,
+)
 from .constants import CcittFaxDecodeParameters as CCITT
 from .constants import ColorSpaces
 from .constants import FilterTypeAbbreviations as FTA
 from .constants import FilterTypes as FT
-from .constants import GraphicsStateParameters as G
 from .constants import ImageAttributes as IA
 from .constants import LzwFilterParameters as LZW
 from .constants import StreamAttributes as SA
@@ -57,13 +62,12 @@ from .generic import (
     NullObject,
 )
 
-if TYPE_CHECKING:
-    try:
-        from typing import Literal  # type: ignore[attr-defined]
-    except ImportError:
-        # PEP 586 introduced typing.Literal with Python 3.8
-        # For older Python versions, the backport typing_extensions is necessary:
-        from typing_extensions import Literal  # type: ignore[misc, assignment]
+try:
+    from typing import Literal, TypeAlias  # type: ignore[attr-defined]
+except ImportError:
+    # PEP 586 introduced typing.Literal with Python 3.8
+    # For older Python versions, the backport typing_extensions is necessary:
+    from typing_extensions import Literal, TypeAlias  # type: ignore[misc, assignment]
 
 
 def decompress(data: bytes) -> bytes:
@@ -147,6 +151,7 @@ class FlateDecode:
                 columns = (
                     1 if decode_parms is None else decode_parms.get(LZW.COLUMNS, 1)
                 )
+                colors = 1 if decode_parms is None else decode_parms.get(LZW.COLORS, 1)
                 bits_per_component = (
                     decode_parms.get(LZW.BITS_PER_COMPONENT, DEFAULT_BITS_PER_COMPONENT)
                     if decode_parms
@@ -155,11 +160,20 @@ class FlateDecode:
 
             # PNG predictor can vary by row and so is the lead byte on each row
             rowlength = (
-                math.ceil(columns * bits_per_component / 8) + 1
+                math.ceil(columns * colors * bits_per_component / 8) + 1
             )  # number of bytes
 
+            # TIFF prediction:
+            if predictor == 2:
+                rowlength -= 1  # remove the predictor byte
+                bpp = rowlength // columns
+                str_data = bytearray(str_data)
+                for i in range(len(str_data)):
+                    if i % rowlength >= bpp:
+                        str_data[i] = (str_data[i] + str_data[i - bpp]) % 256
+                str_data = bytes(str_data)
             # PNG prediction:
-            if 10 <= predictor <= 15:
+            elif 10 <= predictor <= 15:
                 str_data = FlateDecode._decode_png_prediction(str_data, columns, rowlength)  # type: ignore
             else:
                 # unsupported predictor
@@ -173,6 +187,7 @@ class FlateDecode:
         if len(data) % rowlength != 0:
             raise PdfReadError("Image data is not rectangular")
         prev_rowdata = (0,) * rowlength
+        bpp = (rowlength - 1) // columns  # recomputed locally to not change params
         for row in range(len(data) // rowlength):
             rowdata = [
                 ord_(x) for x in data[(row * rowlength) : ((row + 1) * rowlength)]
@@ -182,26 +197,28 @@ class FlateDecode:
             if filter_byte == 0:
                 pass
             elif filter_byte == 1:
-                for i in range(2, rowlength):
-                    rowdata[i] = (rowdata[i] + rowdata[i - 1]) % 256
+                for i in range(bpp + 1, rowlength):
+                    rowdata[i] = (rowdata[i] + rowdata[i - bpp]) % 256
             elif filter_byte == 2:
                 for i in range(1, rowlength):
                     rowdata[i] = (rowdata[i] + prev_rowdata[i]) % 256
             elif filter_byte == 3:
                 for i in range(1, rowlength):
-                    left = rowdata[i - 1] if i > 1 else 0
+                    left = rowdata[i - bpp] if i > bpp else 0
                     floor = math.floor(left + prev_rowdata[i]) / 2
                     rowdata[i] = (rowdata[i] + int(floor)) % 256
             elif filter_byte == 4:
                 for i in range(1, rowlength):
-                    left = rowdata[i - 1] if i > 1 else 0
+                    left = rowdata[i - bpp] if i > bpp else 0
                     up = prev_rowdata[i]
-                    up_left = prev_rowdata[i - 1] if i > 1 else 0
+                    up_left = prev_rowdata[i - bpp] if i > bpp else 0
                     paeth = paeth_predictor(left, up, up_left)
                     rowdata[i] = (rowdata[i] + paeth) % 256
             else:
                 # unsupported PNG filter
-                raise PdfReadError(f"Unsupported PNG filter {filter_byte!r}")
+                raise PdfReadError(
+                    f"Unsupported PNG filter {filter_byte!r}"
+                )  # pragma: no cover
             prev_rowdata = tuple(rowdata)
             output.write(bytearray(rowdata[1:]))
         return output.getvalue()
@@ -600,7 +617,7 @@ def decode_stream_data(stream: Any) -> Union[str, bytes]:  # utils.StreamObject
                 data = ASCII85Decode.decode(data)
             elif filter_type == FT.DCT_DECODE:
                 data = DCTDecode.decode(data)
-            elif filter_type == "/JPXDecode":
+            elif filter_type == FT.JPX_DECODE:
                 data = JPXDecode.decode(data)
             elif filter_type == FT.CCITT_FAX_DECODE:
                 height = stream.get(IA.HEIGHT, ())
@@ -624,7 +641,52 @@ def decodeStreamData(stream: Any) -> Union[str, bytes]:  # deprecated
     return decode_stream_data(stream)
 
 
-def _xobj_to_image(x_object_obj: Dict[str, Any]) -> Tuple[Optional[str], bytes]:
+mode_str_type: TypeAlias = Literal["", "1", "RGB", "P", "L", "RGBA", "CMYK"]
+
+
+def _get_imagemode(
+    color_space: Union[str, List[Any]], color_components: int, prev_mode: mode_str_type
+) -> mode_str_type:
+    """Returns the image mode not taking into account mask(transparency)"""
+    if isinstance(color_space, str):
+        pass
+    elif not isinstance(color_space, list):
+        raise PdfReadError(
+            "can not interprete colorspace", color_space
+        )  # pragma: no cover
+    elif color_space[0] == "/ICCBased":
+        icc_profile = color_space[1].get_object()
+        color_components = cast(int, icc_profile["/N"])
+        color_space = icc_profile.get("/Alternate", "")
+    elif color_space[0] == "/Indexed":
+        color_space = color_space[1].get_object()
+        if isinstance(color_space, list):
+            color_space = color_space[1].get_object()["/Alternate"]
+        color_components = 1 if "Gray" in color_space else 2
+        if not (isinstance(color_space, str) and "Gray" in color_space):
+            color_space = "palette"
+    elif color_space[0] == "/Separation":
+        color_space = color_space[2]
+    elif color_space[0] == "/DeviceN":
+        color_space = color_space[2]
+        color_components = len(color_space[1])
+
+    mode_map = {
+        "1bit": "1",  # 0 will be used for 1 bit
+        "/DeviceGray": "L",
+        "palette": "P",  # reserved for color_components alignment
+        "/DeviceRGB": "RGB",
+        "/DeviceCMYK": "CMYK",
+    }
+    mode: mode_str_type = (
+        mode_map.get(color_space)  # type: ignore
+        or list(mode_map.values())[color_components]
+        or prev_mode
+    )  # type: ignore
+    return mode
+
+
+def _xobj_to_image(x_object_obj: Dict[str, Any]) -> Tuple[Optional[str], bytes, Any]:
     """
     Users need to have the pillow package installed.
 
@@ -635,7 +697,7 @@ def _xobj_to_image(x_object_obj: Dict[str, Any]) -> Tuple[Optional[str], bytes]:
       x_object_obj:
 
     Returns:
-        Tuple[file extension, bytes]
+        Tuple[file extension, bytes, PIL.Image.Image]
     """
     try:
         from PIL import Image
@@ -644,71 +706,99 @@ def _xobj_to_image(x_object_obj: Dict[str, Any]) -> Tuple[Optional[str], bytes]:
             "pillow is required to do image extraction. "
             "It can be installed via 'pip install pypdf[image]'"
         )
+    # for error reporting
+    if (
+        hasattr(x_object_obj, "indirect_reference") and x_object_obj is None
+    ):  # pragma: no cover
+        obj_as_text = x_object_obj.indirect_reference.__repr__()
+    else:
+        obj_as_text = x_object_obj.__repr__()
 
     size = (x_object_obj[IA.WIDTH], x_object_obj[IA.HEIGHT])
     data = x_object_obj.get_data()  # type: ignore
+    colors = x_object_obj.get("/Colors", 1)
+    color_space: Any = x_object_obj.get("/ColorSpace", NullObject()).get_object()
     if (
         IA.COLOR_SPACE in x_object_obj
         and x_object_obj[IA.COLOR_SPACE] == ColorSpaces.DEVICE_RGB
     ):
         # https://pillow.readthedocs.io/en/stable/handbook/concepts.html#modes
-        mode: Literal["1", "RGB", "P", "L", "RGBA"] = "RGB"
-    elif x_object_obj.get("/BitsPerComponent", 8) == 1:
-        mode = "1"
+        mode: mode_str_type = "RGB"
+    if x_object_obj.get("/BitsPerComponent", 8) == 1:
+        mode = _get_imagemode("1bit", 0, "")
     else:
-        mode = "P"
+        mode = _get_imagemode(
+            color_space,
+            2
+            if (
+                colors == 1
+                and (
+                    not isinstance(color_space, NullObject)
+                    and "Gray" not in color_space
+                )
+            )
+            else colors,
+            "",
+        )
     extension = None
-    if SA.FILTER in x_object_obj:
-        if x_object_obj[SA.FILTER] == FT.FLATE_DECODE:
-            extension = ".png"  # mime_type = "image/png"
-            color_space = None
-            if "/ColorSpace" in x_object_obj:
-                color_space = x_object_obj["/ColorSpace"].get_object()
-                if (
-                    isinstance(color_space, ArrayObject)
-                    and color_space[0] == "/Indexed"
-                ):
-                    color_space, base, hival, lookup = (
-                        value.get_object() for value in color_space
-                    )
+    alpha = None
 
+    if SA.FILTER in x_object_obj:
+        if x_object_obj[SA.FILTER] == FT.FLATE_DECODE or x_object_obj[SA.FILTER] == [
+            FT.FLATE_DECODE
+        ]:
+            extension = ".png"  # mime_type = "image/png"
+            if isinstance(color_space, ArrayObject) and color_space[0] == "/Indexed":
+                color_space, base, hival, lookup = (
+                    value.get_object() for value in color_space
+                )
             img = Image.frombytes(mode, size, data)
             if color_space == "/Indexed":
                 from .generic import ByteStringObject
 
                 if isinstance(lookup, ByteStringObject):
-                    if base == ColorSpaces.DEVICE_GRAY and len(lookup) == hival + 1:
-                        lookup = b"".join(
-                            [lookup[i : i + 1] * 3 for i in range(len(lookup))]
+                    try:
+                        nb, conv, mode = {  # type: ignore
+                            "1": (0, "", ""),
+                            "L": (1, "P", "L"),
+                            "P": (0, "", ""),
+                            "RGB": (3, "P", "RGB"),
+                            "CMYK": (4, "P", "CMYK"),
+                        }[_get_imagemode(base, 0, "")]
+                    except KeyError:  # pragma: no cover
+                        logger_warning(
+                            f"Base {base} not coded please share the pdf file with pypdf dev team",
+                            __name__,
                         )
-                    img.putpalette(lookup)
+                        lookup = None
+                    else:
+                        img = img.convert(conv)
+                        if len(lookup) != (hival + 1) * nb:
+                            logger_warning(
+                                f"Invalid Lookup Table in {obj_as_text}", __name__
+                            )
+                            lookup = None
+                        if mode == "L":
+                            # gray lookup does not work : it is converted to a similar RGB lookup
+                            lookup = b"".join([bytes([b, b, b]) for b in lookup])
+                            mode = "RGB"
+                        if lookup is not None:
+                            img.putpalette(lookup, rawmode=mode)
                 else:
                     img.putpalette(lookup.get_data())
                 img = img.convert("L" if base == ColorSpaces.DEVICE_GRAY else "RGB")
-            elif color_space is not None and color_space[0] == "/ICCBased":
+            elif (
+                not isinstance(color_space, NullObject)
+                and color_space[0] == "/ICCBased"
+            ):
                 # see Table 66 - Additional Entries Specific to an ICC Profile
                 # Stream Dictionary
-                icc_profile = color_space[1].get_object()
-                color_components = cast(int, icc_profile["/N"])
-                alternate_colorspace = icc_profile["/Alternate"]
-                color_space = alternate_colorspace
-                mode_map = {
-                    "/DeviceGray": "L",
-                    "/DeviceRGB": "RGB",
-                    "/DeviceCMYK": "RGBA",
-                }
-                mode = (
-                    mode_map.get(color_space)  # type: ignore
-                    or {1: "L", 3: "RGB", 4: "RGBA"}.get(color_components)
-                    or mode
-                )  # type: ignore
-                img = Image.frombytes(mode, size, data)
-            if G.S_MASK in x_object_obj:  # add alpha channel
-                alpha = Image.frombytes("L", size, x_object_obj[G.S_MASK].get_data())
-                img.putalpha(alpha)
-            img_byte_arr = BytesIO()
-            img.convert("RGBA").save(img_byte_arr, format="PNG")
-            data = img_byte_arr.getvalue()
+                mode = _get_imagemode(color_space, colors, mode)
+                extension = ".png"
+                img = Image.frombytes(
+                    mode, size, data
+                )  # reloaded as mode may have change
+            image_format = "PNG"
         elif x_object_obj[SA.FILTER] in (
             [FT.LZW_DECODE],
             [FT.ASCII_85_DECODE],
@@ -719,20 +809,82 @@ def _xobj_to_image(x_object_obj: Dict[str, Any]) -> Tuple[Optional[str], bytes]:
             # extension
             if x_object_obj[SA.FILTER] in [[FT.LZW_DECODE], [FT.CCITT_FAX_DECODE]]:
                 extension = ".tiff"  # mime_type = "image/tiff"
+                image_format = "TIFF"
             else:
                 extension = ".png"  # mime_type = "image/png"
+                image_format = "PNG"
             data = b_(data)
-        elif x_object_obj[SA.FILTER] == FT.DCT_DECODE:
-            extension = ".jpg"  # mime_type = "image/jpeg"
-        elif x_object_obj[SA.FILTER] == "/JPXDecode":
+            img = Image.open(BytesIO(data), formats=("TIFF", "PNG"))
+        elif x_object_obj[SA.FILTER] in [FT.DCT_DECODE, [FT.DCT_DECODE]]:
+            extension = ".jpg"
+            img = Image.open(BytesIO(data))
+            image_format = "JPEG"
+        elif x_object_obj[SA.FILTER] in [FT.JPX_DECODE, [FT.JPX_DECODE]]:
             extension = ".jp2"  # mime_type = "image/x-jp2"
-        elif x_object_obj[SA.FILTER] == FT.CCITT_FAX_DECODE:
+            img1 = Image.open(BytesIO(data), formats=("JPEG2000",))
+            mode = _get_imagemode(color_space, colors, mode)
+            if img1.mode == "RGBA" and mode == "RGB":
+                mode = "RGBA"
+            # we need to convert to the good mode
+            try:
+                if img1.mode != mode:
+                    img = Image.frombytes(mode, img1.size, img1.tobytes())
+                else:
+                    img = img1
+            except OSError:
+                img = Image.frombytes(mode, img1.size, img1.tobytes())
+            # for CMYK conversion :
+            # https://stcom/questions/38855022/conversion-from-cmyk-to-rgb-with-pillow-is-different-from-that-of-photoshop
+            # not implemented for the moment as I need to get properly the ICC
+            if img.mode == "CMYK":
+                img = img.convert("RGB")
+            image_format = "JPEG2000"
+        elif x_object_obj[SA.FILTER] in [FT.CCITT_FAX_DECODE, [FT.CCITT_FAX_DECODE]]:
             extension = ".tiff"  # mime_type = "image/tiff"
+            img = Image.open(BytesIO(data), formats=("TIFF",))
+            image_format = "TIFF"
     else:
         extension = ".png"  # mime_type = "image/png"
         img = Image.frombytes(mode, size, data)
-        img_byte_arr = BytesIO()
-        img.save(img_byte_arr, format="PNG")
-        data = img_byte_arr.getvalue()
+        image_format = "PNG"
 
-    return extension, data
+    if IA.S_MASK in x_object_obj:  # add alpha channel
+        alpha = _xobj_to_image(x_object_obj[IA.S_MASK])[2]
+        if img.size != alpha.size:
+            logger_warning(f"image and mask size not matching: {obj_as_text}", __name__)
+        else:
+            # TODO : implement mask
+            if alpha.mode != "L":
+                alpha = alpha.convert("L")
+            scale = x_object_obj[IA.S_MASK].get("/Decode", [0.0, 1.0])
+            if (scale[1] - scale[0]) != 1.0:
+                alpha = alpha.point(
+                    [
+                        round(255.0 * (v / 255.0 * (scale[1] - scale[0]) + scale[0]))
+                        for v in range(256)
+                    ]
+                )
+            if img.mode == "P":
+                img = img.convert("RGB")
+            img.putalpha(alpha)
+        if "JPEG" in image_format:
+            extension = ".jp2"
+            image_format = "JPEG2000"
+        else:
+            extension = ".png"
+            image_format = "PNG"
+
+    img_byte_arr = BytesIO()
+    try:
+        img.save(img_byte_arr, format=image_format)
+    except OSError:  # pragma: no cover
+        # odd error
+        img_byte_arr = BytesIO()
+        img.save(img_byte_arr, format=image_format)
+    data = img_byte_arr.getvalue()
+
+    try:  # temporary try/except until other fixes of images
+        img = Image.open(BytesIO(data))
+    except Exception:
+        img = None  # type: ignore
+    return extension, data, img
