@@ -38,7 +38,6 @@ from .._utils import (
     StreamType,
     b_,
     deprecation_with_replacement,
-    hex_str,
     hexencode,
     logger_warning,
     read_non_whitespace,
@@ -344,7 +343,7 @@ class IndirectObject(PdfObject):
         r = read_non_whitespace(stream)
         if r != b"R":
             raise PdfReadError(
-                f"Error reading indirect object reference at byte {hex_str(stream.tell())}"
+                f"Error reading indirect object reference at byte {hex(stream.tell())}"
             )
         return IndirectObject(int(idnum), int(generation), pdf)
 
@@ -672,13 +671,11 @@ class NameObject(str, PdfObject):
             raise UnicodeDecodeError("", name, 0, 0, "Code Not Found")
         except (UnicodeEncodeError, UnicodeDecodeError) as e:
             if not pdf.strict:
-                logger_warning(
-                    f"Illegal character in Name Object ({repr(name)})", __name__
-                )
+                logger_warning(f"Illegal character in Name Object ({name!r})", __name__)
                 return NameObject(name.decode("charmap"))
             else:
                 raise PdfReadError(
-                    f"Illegal character in Name Object ({repr(name)})"
+                    f"Illegal character in Name Object ({name!r})"
                 ) from e
 
     @staticmethod
