@@ -1,4 +1,3 @@
-import decimal
 from typing import Any, List, Tuple, Union
 
 from .._utils import deprecation_no_replacement, deprecation_with_replacement
@@ -8,12 +7,15 @@ from ._data_structures import ArrayObject
 
 class RectangleObject(ArrayObject):
     """
-    This class is used to represent *page boxes* in pypdf. These boxes include:
-        * :attr:`artbox <pypdf._page.PageObject.artbox>`
-        * :attr:`bleedbox <pypdf._page.PageObject.bleedbox>`
-        * :attr:`cropbox <pypdf._page.PageObject.cropbox>`
-        * :attr:`mediabox <pypdf._page.PageObject.mediabox>`
-        * :attr:`trimbox <pypdf._page.PageObject.trimbox>`
+    This class is used to represent *page boxes* in pypdf.
+
+    These boxes include:
+
+    * :attr:`artbox <pypdf._page.PageObject.artbox>`
+    * :attr:`bleedbox <pypdf._page.PageObject.bleedbox>`
+    * :attr:`cropbox <pypdf._page.PageObject.cropbox>`
+    * :attr:`mediabox <pypdf._page.PageObject.mediabox>`
+    * :attr:`trimbox <pypdf._page.PageObject.trimbox>`
     """
 
     def __init__(
@@ -46,7 +48,7 @@ class RectangleObject(ArrayObject):
         return self._ensure_is_number(value)
 
     def __repr__(self) -> str:
-        return f"RectangleObject({repr(list(self))})"
+        return f"RectangleObject({list(self)!r})"
 
     @property
     def left(self) -> FloatObject:
@@ -113,7 +115,7 @@ class RectangleObject(ArrayObject):
         return self.bottom
 
     @property
-    def lower_left(self) -> Tuple[decimal.Decimal, decimal.Decimal]:
+    def lower_left(self) -> Tuple[float, float]:
         """
         Property to read and modify the lower left coordinate of this box
         in (x,y) form.
@@ -125,7 +127,7 @@ class RectangleObject(ArrayObject):
         self[0], self[1] = (self._ensure_is_number(x) for x in value)
 
     @property
-    def lower_right(self) -> Tuple[decimal.Decimal, decimal.Decimal]:
+    def lower_right(self) -> Tuple[float, float]:
         """
         Property to read and modify the lower right coordinate of this box
         in (x,y) form.
@@ -137,7 +139,7 @@ class RectangleObject(ArrayObject):
         self[2], self[1] = (self._ensure_is_number(x) for x in value)
 
     @property
-    def upper_left(self) -> Tuple[decimal.Decimal, decimal.Decimal]:
+    def upper_left(self) -> Tuple[float, float]:
         """
         Property to read and modify the upper left coordinate of this box
         in (x,y) form.
@@ -149,7 +151,7 @@ class RectangleObject(ArrayObject):
         self[0], self[3] = (self._ensure_is_number(x) for x in value)
 
     @property
-    def upper_right(self) -> Tuple[decimal.Decimal, decimal.Decimal]:
+    def upper_right(self) -> Tuple[float, float]:
         """
         Property to read and modify the upper right coordinate of this box
         in (x,y) form.
@@ -162,25 +164,25 @@ class RectangleObject(ArrayObject):
 
     def getLowerLeft(
         self,
-    ) -> Tuple[decimal.Decimal, decimal.Decimal]:  # deprecated
+    ) -> Tuple[float, float]:  # deprecated
         deprecation_with_replacement("getLowerLeft", "lower_left", "3.0.0")
         return self.lower_left
 
     def getLowerRight(
         self,
-    ) -> Tuple[decimal.Decimal, decimal.Decimal]:  # deprecated
+    ) -> Tuple[float, float]:  # deprecated
         deprecation_with_replacement("getLowerRight", "lower_right", "3.0.0")
         return self.lower_right
 
     def getUpperLeft(
         self,
-    ) -> Tuple[decimal.Decimal, decimal.Decimal]:  # deprecated
+    ) -> Tuple[float, float]:  # deprecated
         deprecation_with_replacement("getUpperLeft", "upper_left", "3.0.0")
         return self.upper_left
 
     def getUpperRight(
         self,
-    ) -> Tuple[decimal.Decimal, decimal.Decimal]:  # deprecated
+    ) -> Tuple[float, float]:  # deprecated
         deprecation_with_replacement("getUpperRight", "upper_right", "3.0.0")
         return self.upper_right
 
@@ -201,65 +203,57 @@ class RectangleObject(ArrayObject):
         self[2], self[3] = (self._ensure_is_number(x) for x in value)
 
     @property
-    def width(self) -> decimal.Decimal:
+    def width(self) -> float:
         return self.right - self.left
 
-    def getWidth(self) -> decimal.Decimal:  # deprecated
+    def getWidth(self) -> float:  # deprecated
         deprecation_with_replacement("getWidth", "width", "3.0.0")
         return self.width
 
     @property
-    def height(self) -> decimal.Decimal:
+    def height(self) -> float:
         return self.top - self.bottom
 
-    def getHeight(self) -> decimal.Decimal:  # deprecated
+    def getHeight(self) -> float:  # deprecated
         deprecation_with_replacement("getHeight", "height", "3.0.0")
         return self.height
 
     @property
-    def lowerLeft(self) -> Tuple[decimal.Decimal, decimal.Decimal]:  # deprecated
+    def lowerLeft(self) -> Tuple[float, float]:  # deprecated
         deprecation_with_replacement("lowerLeft", "lower_left", "3.0.0")
         return self.lower_left
 
     @lowerLeft.setter
-    def lowerLeft(
-        self, value: Tuple[decimal.Decimal, decimal.Decimal]
-    ) -> None:  # deprecated
+    def lowerLeft(self, value: Tuple[float, float]) -> None:  # deprecated
         deprecation_with_replacement("lowerLeft", "lower_left", "3.0.0")
         self.lower_left = value
 
     @property
-    def lowerRight(self) -> Tuple[decimal.Decimal, decimal.Decimal]:  # deprecated
+    def lowerRight(self) -> Tuple[float, float]:  # deprecated
         deprecation_with_replacement("lowerRight", "lower_right", "3.0.0")
         return self.lower_right
 
     @lowerRight.setter
-    def lowerRight(
-        self, value: Tuple[decimal.Decimal, decimal.Decimal]
-    ) -> None:  # deprecated
+    def lowerRight(self, value: Tuple[float, float]) -> None:  # deprecated
         deprecation_with_replacement("lowerRight", "lower_right", "3.0.0")
         self.lower_right = value
 
     @property
-    def upperLeft(self) -> Tuple[decimal.Decimal, decimal.Decimal]:  # deprecated
+    def upperLeft(self) -> Tuple[float, float]:  # deprecated
         deprecation_with_replacement("upperLeft", "upper_left", "3.0.0")
         return self.upper_left
 
     @upperLeft.setter
-    def upperLeft(
-        self, value: Tuple[decimal.Decimal, decimal.Decimal]
-    ) -> None:  # deprecated
+    def upperLeft(self, value: Tuple[float, float]) -> None:  # deprecated
         deprecation_with_replacement("upperLeft", "upper_left", "3.0.0")
         self.upper_left = value
 
     @property
-    def upperRight(self) -> Tuple[decimal.Decimal, decimal.Decimal]:  # deprecated
+    def upperRight(self) -> Tuple[float, float]:  # deprecated
         deprecation_with_replacement("upperRight", "upper_right", "3.0.0")
         return self.upper_right
 
     @upperRight.setter
-    def upperRight(
-        self, value: Tuple[decimal.Decimal, decimal.Decimal]
-    ) -> None:  # deprecated
+    def upperRight(self, value: Tuple[float, float]) -> None:  # deprecated
         deprecation_with_replacement("upperRight", "upper_right", "3.0.0")
         self.upper_right = value
