@@ -90,7 +90,7 @@ def read_string_from_stream(
                         if b"0" <= ntok and ntok <= b"7":
                             tok += ntok
                         else:
-                            stream.seek(-1, 1)  # ntok has to be analysed
+                            stream.seek(-1, 1)  # ntok has to be analyzed
                             break
                     tok = b_(chr(int(tok, base=8)))
                 elif tok in b"\n\r":
@@ -104,7 +104,7 @@ def read_string_from_stream(
                     # line break was escaped:
                     tok = b""
                 else:
-                    msg = rf"Unexpected escaped string: {tok.decode('utf8')}"
+                    msg = f"Unexpected escaped string: {tok.decode('utf-8','ignore')}"
                     logger_warning(msg, __name__)
         txt.append(tok)
     return create_string_object(b"".join(txt), forced_encoding)
@@ -118,15 +118,14 @@ def create_string_object(
     Create a ByteStringObject or a TextStringObject from a string to represent the string.
 
     Args:
-      string:
-      forced_encoding:
+        string: The data being used
+        forced_encoding: Typically None, or an encoding string
 
     Returns:
-      A ByteStringObject
+        A ByteStringObject
 
     Raises:
-      TypeError: If string is not of type str or bytes.
-
+        TypeError: If string is not of type str or bytes.
     """
     if isinstance(string, str):
         return TextStringObject(string)
